package de.kif.backend

import mu.KotlinLogging
import java.io.File
import java.net.URI
import java.nio.file.*

/**
 * Manage static resource files of the ui
 */
object Resources {

    private val logger = KotlinLogging.logger {}

    /**
     * Extract web directory from jar file.
     *
     * @return File that points the extracted web directory.
     */
    private fun extractWebFolder(): File {
        val destination: Path = Files.createTempDirectory("web")

        val classPath = "web"
        val uri: URI = this::class.java.classLoader.getResource(classPath).toURI()
        val fileSystem: FileSystem?
        val src: Path = if (uri.scheme == "jar") {
            fileSystem = FileSystems.newFileSystem(uri, mutableMapOf<String, Any?>())
            fileSystem.getPath(classPath)
        } else {
            Paths.get(uri)
        }

        Files.walk(src).forEach { path: Path ->
            val name = path.toAbsolutePath().toString().let { name ->
                name.drop(classPath.length + name.indexOf(classPath, ignoreCase = true)).dropWhile {
                    it == '/' || it == '\\'
                }
            }
            if (Files.isDirectory(path)) {
                Files.createDirectories(destination.resolve(name))
            } else {
                Files.copy(path, destination.resolve(name))
            }
        }

        return destination.toFile()
    }

    /**
     * File that points the web directory.
     */
    val directory = File(".").absoluteFile.canonicalFile.let { currentDir ->
        listOf(
            "web",
            "../web"
        ).map {
            File(currentDir, it)
        }.firstOrNull { it.isDirectory }?.absoluteFile?.canonicalFile ?: extractWebFolder()
    }.also {
        logger.info { "Web directory: $it" }
    }

    /**
     * List of js modules to be included.
     */
    val jsModules = directory.list().toList().filter {
        it.endsWith(".js") &&
                !it.endsWith(".meta.js") &&
                !it.endsWith("-test.js") &&
                it != "require.min.js"
    }.joinToString(", ") { "'${it.take(it.length - 3)}'" }

}
